; bootstrap.asm for lightos.ld
; Written by Kangmo Kim at birthday of lightOS, 02/01/2004

	%include 'md/i686/defs.inc'

;%imacro	PUTBYTE16 1
;	push dx
;		mov dh, %1
;		call put_byte16_
;		mov dl, ' '
;		call putc16
;	pop  dx
;%endmacro

;%imacro	PUTWORD16 1
;	push dx
;		mov dx, %1
;		call put_word16_
;		mov dl, ' '
;		call putc16
;	pop  dx
;%endmacro



; Address Map
;
; 0x7C00 ~ 0x7FFF (512) : bootstrap.o ; start@bootstrap.o=0x7C00
; 0x8000 ~ 0x8FFF (64K) : temporary stack for bootstrap.o  
;                         unused after switching to protected mode
; 0x9000 ~ 0x9FFF (64K) : entry16.o ; entry16@entry16.o=0x9000
; 0xA000 ~              : entry32.o and object files compiled from C sources 
;                         entry32@entry32.o=0xA000
;

CURSOR_QUERY_PORT	equ 03D4h
CURSOR_DATA_PORT	equ 03D5h
CURSOR_OFFSET_MSB	equ 14
CURSOR_OFFSET_LSB	equ 15

SCREEN_MASK         equ 0Fh
	
SECTOR_SIZE					equ 512
START_ADDRESS				equ	7C00h

; stack pointer will be initialized at 0x0800:0x0FFF ( 0x8FFF )
; the stack will be located at 0x8000 ~ 0x8FFF (64K)
STACK_SEG                   equ 0800h
STACK_OFF                   equ 0FFFh
	
START_SEG_OF_SECTOR2		equ	0900h
START_OFF_OF_SECTOR2		equ  0000h
START_ADDRESS_OF_SECTOR2	equ  9000h
;KERNEL_SIZE				equ 70000
;KERNEL_SIZE_IN_SECTORS		equ KERNEL_SIZE / SECTOR_SIZE
;KERNEL_SIZE_IN_SECTORS     equ 100
;KERNEL_SIZE_IN_SECTORS     equ 63
KERNEL_SIZE_IN_SECTORS      equ 100
SECTORS_PER_TRACK           equ 18

; NOTE ! 
; Bochs emulator supports 8 sectors read across cylinder or header boundary
; But BIOS does not support it. You have to read only one sectors at once
SECTORS_TO_READ             equ 1

ERR_DRIVE_NOT_READY			equ 0AAh


[ORG START_ADDRESS]

[BITS 16]	; start with Real Mode

start:
    cli
	; initialize segment registers
	mov ax, 0
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax
	
	; initialize stack pointer
	mov ax, STACK_SEG
	mov ss, ax
	mov sp, STACK_OFF
	sti
	
	call clear_scr
	
	mov bx, str_loading_bootsec
	call puts16
	jmp reset_drive

int13_error_retry:
	call display_int13_error
reset_drive:
	;-----------------------------------------------------------------------
	; Reset Drive
	;-----------------------------------------------------------------------
	
	mov ah, 0			; [int13] reset drive
	int 13h
	
	cmp ah, 0
	jne int13_error_retry	; ah == 0 if success otherwise error.

	;-----------------------------------------------------------------------
	; Load the second sector at
	;-----------------------------------------------------------------------
read_drive:

	;-----------------------------------------------------------------------
	; 1.44 MB floppy
	; Total sectors = Tracks ( cylinders ) * heads * tracks per sectors
	;               = 80 * 2 * 18  = 2880
	;
	; The order of data written in C/H/S
	; 
	; (C/H/S)
	;
	; (0/0/1) ~ (0/0/18)
	; (0/1/1) ~ (0/1/18)
	; (1/0/1) ~ (1/0/18)
	; (1/1/1) ~ (1/1/18)
	; (2/0/1) ~ (2/0/18)
	; ....
	;-----------------------------------------------------------------------
	; log_sec_num	es
	; -----------   ------
	; 0         	0x0000 ( 0x000:0000 => 0 )
	; 1         	0x0020 ( 0x020:0000 => 512 )
	; 2         	0x0040 ( 0x040:0000 => 1024 )
	 

	; set es:bx to the memory location where the sector is loaded.
	mov ax, word [log_sec_num]
cont_load :	
	shl ax, 5			; To turn 1 into 0x20, 2 into 0x40 
						; I need to multiply 32 to log_sec_num to get es.
	add ax, START_SEG_OF_SECTOR2
	mov	es, ax
	
	; note that sector starts from 1, not 0

	; cylinder = (( log_sec_num + 01 ) / SECTORS_PER_TRACK ) / 2
	; head     = (( log_sec_num + 01 ) / SECTORS_PER_TRACK ) % 2
	; sector   = (( log_sec_num + 01 ) % SECTORS_PER_TRACK ) + 1
	mov ax, word [log_sec_num]
	add ax, 1
	mov bl, SECTORS_PER_TRACK
	div bl				

	mov ch,	al 			; Disk cylinder
	shr ch, 1			; /= 2

	mov dh, al			; Disk head
	and dh, 1			; %= 2

	mov cl, ah			; Disk sector (starts with 1, not 0 )
	add cl, 1			; += 1

	mov ah, 02h			; [int13] function code ; load sector into es:bx

	mov bx, START_OFF_OF_SECTOR2    ; set es:bx - bx(offset) is always zero
	
	mov al, SECTORS_TO_READ			; Number of sectors to read 
	mov dl, 0						; Disk drive ( 0 => A: )

    ; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
	; print for debugging
	; (2 bytes) logical sector#
	; (2 bytes) es
	; (1 bytes) head #
	; (1 bytes) drive #
	; (1 bytes) cylinder #
	; (1 bytes) sector #
    ; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
;	push bx
;		mov bx, str_sep			; '|'
;		call puts16
;	pop bx
			
;	push dx
;		mov  dx, word [log_sec_num]   ; logical sector #
;		call put_word16
;		mov  dx, es			; es
;		call put_word16
;	pop dx
		
;	push dx
;		call put_byte16      ; dh - head #
;		mov dh, dl           ; dl - drive	
;		call put_byte16	    
;		mov  dh, ch         ; al - cylinder # 
;		call put_byte16
;		mov  dh, cl         ; cl - sector #
;		call put_byte16
;	pop dx

;	PUTCHAR16 'X'
	
	int 13h
	
;	PUTCHAR16 'O'


	cmp ah, 0
	jne int13_error_retry	; ah == 0 if success otherwise error.

	mov ax, word [log_sec_num]
	add ax, SECTORS_TO_READ		; 1 sector has been read.


	mov word [log_sec_num], ax
	cmp ax, KERNEL_SIZE_IN_SECTORS
	
	jl  read_drive      ; yet we have to read more sectors
	
	; jump to entry in entry.asm
	jmp START_SEG_OF_SECTOR2 : START_OFF_OF_SECTOR2

; logical sector number which starts from 0 ~ n
; logical sector 0 is mapped to (C/H/S) = (0/0/1)
log_sec_num:
	dw 0
str_loading_bootsec
	db 'Loading Boot Image.',0
str_int_13_error:
    db 'int13 err#',0
str_at_log_sector:
	db 'lsec#',0
hexa_chars:
	db '0123456789ABCDEF' 
str_sep:
	db '|',0
	
; [OUT] bx : cursor offset to VMEM
getcursoroffset16:
	push ax
	push dx

		mov dx, CURSOR_QUERY_PORT
		mov al, CURSOR_OFFSET_MSB
		out dx, al
	
		mov dx, CURSOR_DATA_PORT 
		in  al, dx
		
		mov bh, al	; get MSB
	
		mov dx, CURSOR_QUERY_PORT
		mov al, CURSOR_OFFSET_LSB
		out dx, al
		
		mov dx, CURSOR_DATA_PORT
		in  al, dx
		 
		mov bl, al  ; get LSB
		
		shl bx, 1   ; make it adressable to 0xB8000h
	
	pop dx
	pop ax
	
	ret
	
; [IN] bx : cursor offset to VMEM
setcursoroffset16:
	push ax
	push dx
		; convert offset on VMEM to cursor offset
		shr bx, 1

		mov dx, CURSOR_QUERY_PORT
		mov al, CURSOR_OFFSET_MSB
		out dx, al
	
		mov dx, CURSOR_DATA_PORT 
		mov al, bh	; set MSB
		out dx, al
		
	
		mov dx, CURSOR_QUERY_PORT
		mov al, CURSOR_OFFSET_LSB
		out dx, al
		
		mov dx, CURSOR_DATA_PORT
		mov al, bl	; set LSB
		out dx, al
	pop dx
	pop ax
	
	ret
	
	; [IN] dl : character
putc16:	
	push bx
	push cx
		call getcursoroffset16	; bx := cursor offset
		
		push bx 
			add bx, 08000h			; convert to vram ptr
			
			push ds					; change ds to VRAM DS
			
				mov cx, 0B000h
				mov ds, cx
				
				mov byte [bx], dl	; actually put character
				inc bx

				; 0Fh Non blinking White Char On Black
				mov byte [bx], SCREEN_MASK 
	
			pop ds
		pop bx					; change bx back to cursor offset
		add bx, 2				; we put a char which is two bytes of VRAM
		call setcursoroffset16	; cursor offset := bx
		
	pop cx
	pop bx
	ret
	
	
	; [IN ] bx : string address
puts16:
    push bx	; string address, cursor offset on VMEM
    push cx ; temp
    push dx ; character to display
put_next_char:		
		mov dl, byte [bx]		; get character to display
		cmp dl, 0				; end of string ?
		je  end_puts16		

		call putc16
		
		inc bx					; move to next character
		jmp put_next_char
		
end_puts16:		
		
	pop dx
	pop cx
	pop bx
	
	ret
	
; [IN] DH ; word to print
put_byte16 :
	push ax
	push bx
	push dx
	
		mov ah, 0
		mov al, dh
		shr al, 4
		mov bx, hexa_chars
		add bx, ax
		mov dl, byte[ bx ]
		call putc16
	
		mov ah, 0
		mov al, dh
		and al, 0Fh
		mov bx, hexa_chars
		add bx, ax
		mov dl, byte[ bx ]
		call putc16
	
	pop dx
	pop bx
	pop ax
	
	ret

; IN [DX] ; word to print
put_word16 :
	push ax
		; print high word dh
		call put_byte16
		mov dh, dl			; print low word
		call put_byte16
	pop  dx
	
	ret

clear_scr :
		mov bx, 0
		call setcursoroffset16
		mov cx, 25 * 80 
putspace:
		mov dl, 0
		call putc16
		
		dec cx
		cmp cx, 0
		jne putspace
		
		mov bx, 0
		call setcursoroffset16
	
	ret

; [IN] AH - error code
display_int13_error:
;	cmp ah, ERR_DRIVE_NOT_READY
;	je dont_display

	push bx	
		mov bx, str_int_13_error
		call puts16
	
     	mov dh, ah	; set error code to dh
     	call put_byte16
	
     	mov bx, str_at_log_sector
     	call puts16
	
    	mov dx, [log_sec_num]
     	call put_word16
	pop bx

dont_display:
	; display "."
;	mov dl, '.'
;	call putc16
	ret
	   
; int 13 error codes	
;    case 0x00: msg("Operation Successful");    break;
;    case 0x01: msg("Bad command");     break;
;    case 0x02: msg("Address mark not found");    break;
;    case 0x03: msg("Attempt to write on write-protected disk"); break;
;    case 0x04: msg("Sector not found");    break;
;    case 0x05: msg("Reset failed (hard disk)");   break;
;    case 0x06: msg("Disk changed since last operation");  break;
;    case 0x07: msg("Drive parameter activity failed");   break;
;    case 0x08: msg("DMA overrun");     break;
;    case 0x09: msg("Attempt to DMA across 64K boundary");  break;
;    case 0x0A: msg("Bad sector detected");    break;
;    case 0x0B: msg("Bad track detected");    break;
;    case 0x0C: msg("Unsupported track");    break;
;    case 0x10: msg("Bad CRC/ECC on disk read");   break;
;    case 0x11: msg("CRC/ECC corrected data error");   break;
;    case 0x20: msg("Controller has failed");    break;
;    case 0x40: msg("Seek operation failed");    break;
;    case 0x80: msg("Attachment failed to respond");   break;
;    case 0xAA: msg("Drive not ready (hard disk only");   break;
;    case 0xBB: msg("Undefined error occurred (hard disk only)"); break;
;    case 0xCC: msg("Write fault occurred");    break;
;    case 0xE0: msg("Status error");     break;
;    case 0xFF: msg("Sense operation failed");    break;


	times 510-($-$$) db 0		; Fill up the file with zero
	dw 0AA55h					; Magic value that 
								; BIOS checks if this is boot sector
