#include <includes.h>
#include "cpu.h"
#include "msr.h"
#include <thread.h>

/* Standard macro to see if a specific flag is changeable */
static inline int flag_is_changeable_p(u32 flag)
{
	u32 f1, f2;

	asm("pushfl\n\t"
	    "pushfl\n\t"
	    "popl %0\n\t"
	    "movl %0,%1\n\t"
	    "xorl %2,%0\n\t"
	    "pushl %0\n\t"
	    "popfl\n\t"
	    "pushfl\n\t"
	    "popl %0\n\t"
	    "popfl\n\t"
	    : "=&r" (f1), "=&r" (f2)
	    : "ir" (flag));

	return ((f1^f2) & flag) != 0;
}


/* Probe for the CPUID instruction */
int have_cpuid_p(void)
{
	return flag_is_changeable_p(X86_EFLAGS_ID);
}


/*
CPUID EAX=0
EBX "Genu"
EDX "ntel"
ECX "ineI"

CPUID EAX=1
EAX	Version Information (Type, Family, Model, and Stepping ID)
EBX Bits 7-0: Brand Index
	Bits 15-8: CLFLUSH line size. (Value ? 8 = cache line size in bytes)
	Bits 23-16: Number of logical processors per physical processor.
	Bits 31-24: Local APIC ID
ECX Extended Feature Information (see Figure 3-4 and Table 3-10)
EDX Feature Information (see Figure 3-5 and Table 3-11)
*/


void identify_cpu(struct cpu_info_t * c)
{
	u32 tfms;

	if (have_cpuid_p()) {
		/* Get vendor name */
		cpuid(0x00000000, &c->cpuid_level,
		      (s32 *)&c->x86_vendor_id[0],
		      (s32 *)&c->x86_vendor_id[8],
		      (s32 *)&c->x86_vendor_id[4]);
		
		c->x86_vendor_id[12] = '\0';
		
		/* Initialize the standard set of capabilities */
		/* Note that the vendor-specific code below might override */
	
		/* Intel-defined flags: level 0x00000001 */
		if ( c->cpuid_level >= 0x00000001 ) {
			
			cpuid(0x00000001, &tfms, (u32*) & c->ebx, & c->excaps, & c->caps );

			c->x86 = (tfms >> 8) & 15;
			c->x86_model = (tfms >> 4) & 15;
			if (c->x86 == 0xf) {
				c->x86 += (tfms >> 20) & 0xff;
				c->x86_model += ((tfms >> 16) & 0xF) << 4;
			} 
			c->x86_mask = tfms & 15;
			
		} else {
			/* Have CPUID level 0 only - unheard of */
			c->x86 = 4;
		}
	}
}

