/*
 * cpu_feature.h
 *
 * Defines x86 CPU feature bits
 */

#ifndef _CPU_FEATURE_H_
#define _CPU_FEATURE_H_

/* Intel-defined CPU features, CPUID level 0x00000001 (edx) */
#define X86_CAP_FPU		(1 <<  0) /* Onboard FPU */
#define X86_CAP_VME		(1 <<  1) /* Virtual Mode Extensions */
#define X86_CAP_DE		(1 <<  2) /* Debugging Extensions */
#define X86_CAP_PSE 	(1 <<  3) /* Page Size Extensions */
#define X86_CAP_TSC		(1 <<  4) /* Time Stamp Counter */
#define X86_CAP_MSR		(1 <<  5) /* Model-Specific Registers, RDMSR, WRMSR */
#define X86_CAP_PAE		(1 <<  6) /* Physical Address Extensions */
#define X86_CAP_MCE		(1 <<  7) /* Machine Check Architecture */
#define X86_CAP_CX8		(1 <<  8) /* CMPXCHG8 instruction */
#define X86_CAP_APIC	(1 <<  9) /* Onchip APIC */
#define X86_CAP_SEP		(1 << 11) /* SYSENTER/SYSEXIT */
#define X86_CAP_MTRR	(1 << 12) /* Memory Type Range Registers */
#define X86_CAP_PGE		(1 << 13) /* Page Global Enable */
#define X86_CAP_MCA		(1 << 14) /* Machine Check Architecture */
#define X86_CAP_CMOV	(1 << 15) /* CMOV instruction (FCMOVCC and FCOMI too if FPU present) */
#define X86_CAP_PAT		(1 << 16) /* Page Attribute Table */
#define X86_CAP_PSE36	(1 << 17) /* 36-bit PSEs */
#define X86_CAP_PN		(1 << 18) /* Processor serial number */
#define X86_CAP_CLFLSH	(1 << 19) /* Supports the CLFLUSH instruction */
#define X86_CAP_DTES	(1 << 21) /* Debug Trace Store */
#define X86_CAP_ACPI	(1 << 22) /* ACPI via MSR */
#define X86_CAP_MMX		(1 << 23) /* Multimedia Extensions */
#define X86_CAP_FXSR	(1 << 24) /* FXSAVE and FXRSTOR instructions (fast save and restore */
				          /* of FPU context), and CR4.OSFXSR available */
#define X86_CAP_XMM		(1 << 25) /* Streaming SIMD Extensions */
#define X86_CAP_XMM2	(1 << 26) /* Streaming SIMD Extensions-2 */
#define X86_CAP_SELFSNOOP	(1 << 27) /* CPU self snoop */
#define X86_CAP_HT		(1 << 28) /* Hyper-Threading */
#define X86_CAP_ACC		(1 << 29) /* Automatic clock control */
#define X86_CAP_IA64	(1 << 30) /* IA-64 processor */
#define X86_CAP_PBE		(1 << 31) /* Pending Break Enable */

/* Intel-defined CPU features, CPUID level 0x00000001 (ecx) */
#define X86_EXCAP_EST		(1 << 7) /* Enhanced SpeedStep */
#define X86_EXCAP_MWAIT		(1 << 3) /* Monitor/Mwait support */

#endif /* _CPU_FEATURE_H_ */
