	%include 'md/i686/defs.inc'

; 	Every CPU including BSP, APs pass through entry16
;   , which switches to protected mode using temporary GDT
	
	extern kernel_main
	extern init_registers

DS_MAGIC					equ 0xFCED8790

; entry16.o begins at 0x9000. 
; see bootstrap.asm for more information.
ENTRY16_SEG					equ 0000h
ENTRY16_OFF					equ  9000h
ENTRY16_ADDR				equ 09000h
ENTRY16_SIZE				equ 01000h
ENTRY32_ADDR				equ 0A000h

[ORG ENTRY16_ADDR ]

[BITS 16]	; start with Real Mode

entry16 :
	;---------------------------------------------------------------
	; Set DS
	;---------------------------------------------------------------
	mov ax, ENTRY16_SEG	; set ds 
	mov	ds, ax			; lgdt will load gdt from ds:address

	;---------------------------------------------------------------
	; Test DS
	;---------------------------------------------------------------
	mov	 eax, dword [ds_magic ]
	cmp  eax, DS_MAGIC
	je  ds_ok
	
	; Something is wrong
	PUTCHAR16 '1'
	jmp $
ds_ok:
	;---------------------------------------------------------------
	; Load GDT 
	; This GDT is only for getting into protected mode.
	; Real GDT will be loaded in entry32.o
	;---------------------------------------------------------------
	lgdt [ gdt_desc ] 

switch_pmode:
	;---------------------------------------------------------------
	; Switch into Protected mode by setting bit 0 of cr0
	;---------------------------------------------------------------
	
	mov eax, cr0		; copy cr0 to eax for modification
	or  eax, 1			; set bit 0
	mov cr0, eax		; copy modified cr0 value to cr0
	
	;---------------------------------------------------------------
	; Currently we have 16 bit instructions in pipeline.
	; Clear pipeline by doing far jump.
	;---------------------------------------------------------------
	; jmp to entry32 @ entry32.asm
	; ld sets entry32 to ENTRY32_ADDR
	
	jmp KERNEL_CS:ENTRY32_ADDR

ds_magic:
	dd DS_MAGIC
;---------------------------------------------------------------
; GDT
;---------------------------------------------------------------
; These GDT entries are just for getting into protected mode.
; Real GDT is set from entrt32.asm
; 
; Note that KERNEL_CS and KERNEL_DS should be same 
; between entry16 and entry32.
;
; Any modification to this temporary GDT should be propagated to
; real GDT at entry32.o
;
ALIGN 8	
gdt:					; adress of GDT	

gdt_null:				; Null Segment 
	dd 0				; The first Segment is reserved by CPU
	dd 0				; Initialize it with 0

KERNEL_CS 	equ	$-gdt
	dw 0FFFFh			; Limit 0:15
	dw 0				; Base 0:15
	
	db 0				; Base 16:23
	db 10011010b		; [8-12] Type
						;	[8]   - 0 : access by CPU flag
						;	[9]   - 1 : readability of this segment
						;	[10]  - 0 : conforming 
						;				( if set, allow unprivileged access )
						;   [11]  - 1 : code(1), data(0)
						;   [12]  - 1 : code or data (1), otherwise (0)
						; [13-14] - 00 : privileged level
						; [15]    - 1 : present in memory

	db 11001111b		; [16-19] - 1111 : Limit 16:19
						; [20]    - 0 : reserved by OS
						; [21]	  - 0 : reserved by CPU
						; [22]    - 1 : 32bit (1), 16bit(0)
						; [23]    - 1 : Granuality - 4KB(1), 1byte(0)
	
	db 0				; Base 24:31
	
KERNEL_DS 	equ	$-gdt
	dw 0FFFFh			; Limit 0:15
	dw 0				; Base 0:15
	
	db 0				; Base 16:23
	db 10010010b		; [8-12] Type
						;	[8]   - 0 : access by CPU flag
						;	[9]   - 1 : writablity of this segment
						;	[10]  - 0 : expand direction - up(1), down(0)
						;   [11]  - 1 : code(1), data(0)
						;   [12]  - 1 : code or data (1), otherwise (0)
						; [13-14] - 00 : privileged level
						; [15]    - 1 : present in memory
						
	db 11001111b		; same to PMODE_CS
	
	db 0				; Base 24:31
gdt_end:

; Intel IA32 Software Developers Manual says that Aligning GDT descriptor
; to 4 byte gives performance benefits.
ALIGN 4
gdt_desc:					; The GDT descriptor
	dw gdt_end - gdt - 1	; Limit - size of GDT
	dd gdt					; Adress of the GDT

	
	times ENTRY16_SIZE - ($-$$) db 0 	; Fill up file with zero