	%include 'md/i686/defs.inc'
	%include 'md/i686/thread.inc'
	%include 'include/config.inc'

%imacro spin_lock 2
try_lock:	
	TRY_SPIN_LOCK %1, %2
	cmp %2, 0
	jne locked
	pause
	jmp try_lock
locked:
%endmacro

; %1 memory location
%imacro spin_unlock 1
	mov %1, 0
%endmacro


; [ERROR]  stack pointer := 090000h ( beginning at 640KB - 64KB )
; [BSP]  stack pointer := 08C000h
; [AP1]  stack pointer := 08A000h
; [AP2]  stack pointer := 08800hh
; .......

STACK_POINTER_ADDRESS			equ		090000h
STACK_SIZE						equ		002000h 	; ( 8KB )
DS_MAGIC						equ		0xFCED8790

SECTION .text

[BITS 32]				; from now on, we need 32-bit instructions
	global entry32, cpu_count
	extern kernel_main, printk, init_console, __bss_start__, __bss_end__

entry32:
	;---------------------------------------------------------------
	; Initialize registers.
	; Cs already has been initialized by far jump shown above.
	;---------------------------------------------------------------

	; Data Segment selector value of GDT in entry16, which should be
	; same to GDT in this file

	mov eax, KERNEL_DS	
	
	; Initialize all segment registers to Data Segment 
	mov	ds,ax

	;---------------------------------------------------------------
	; Load Real GDT	
	;---------------------------------------------------------------
	lgdt [ gdt_desc ]
	
	mov	es,ax
	mov	fs,ax
	mov	gs,ax
	mov	ss,ax


;						call init_console

	;---------------------------------------------------------------
	; Multiple APs are concurrently executing this routine.
	; Reading / Writing shared memory locations should be spin locked
	;---------------------------------------------------------------
	
;	SPIN_LOCK dword[ cpu_lock ], eax

		;
		; Calculate stack pointer for this CPU
		;
		sub dword [ stack_pointer ], STACK_SIZE
	
		; Get stack pointer for this CPU
		;
		mov eax, dword [ stack_pointer ]
		mov esp, eax 

		; Get CPU ID for this CPU
		;
		mov eax, dword [ cpu_count ]
		inc dword [ cpu_count ]

;	SPIN_UNLOCK dword[ cpu_lock ]

	; Check if the maximum CPU exceeded.
	cmp eax, MAX_CPUS
	jl max_cpu_not_execeeded

	; The number of CPU has execeeded	
	; Console had been already initialized by BSP
	; On error, use STACK_POINTER_ADDRESS which is not used by any CPU.
	mov esp, STACK_POINTER_ADDRESS
	
	push dword MAX_CPUS
	push dword [cpu_count]
	push dword str_cpu_is_unusable
	call printk
	add  esp, 12
	
	jmp $

max_cpu_not_execeeded :

	; fill thread_info_t at the lowest address of stack ( Stack Top )
	; current_thread_info() finds thread_info_t at the TOP of stack
	
	sub esp, STACK_SIZE

	; eax has cpu ID
	
	; Set CPU ID on thread_info_t
	; thread_info_t is at the bottom of stack
	;
	mov dword [ esp + THREAD_INFO_MAGIC ], THREAD_INFO_MAGIC_VALUE
	mov dword [ esp + THREAD_INFO_CPU ], eax
	mov dword [ esp + THREAD_INFO_IRQ_DEPTH ], 0
	mov dword [ esp + THREAD_INFO_IRQ_COUNT ], 0

	add esp, STACK_SIZE
	
;						push dword [8C000h]
;						push esp
;						push eax
;						push str_debug
;						call printk
;						add esp, 16

	; test if DS is set correctly
	mov	 eax, dword [ds_magic]
	cmp  eax, DS_MAGIC
	je   ds_ok
	
	; Something io wrong
	PUTCHAR '3'
	jmp $

;str_debug :
;db 'eax: %x, esp: %x, [8C000h]:%x',CR, 0
;str_debug_value :
;db 'value: %x', CR, 0
ds_ok:	

	;
	; Only BSP clears BSS
	;
	cmp dword[cpu_count], 1
	jne start_kerenel
	;----------------------------------------------------------------
	; zero the BSS ( global variables that is not initialized )
	;----------------------------------------------------------------
	mov edi,__bss_start__
	mov ecx,__bss_end__
	sub ecx,edi
	xor eax,eax
	rep stosb

start_kerenel:	
	;----------------------------------------------------------------
	; jump into kernel_main() function in kernel.o
	;----------------------------------------------------------------
	jmp kernel_main		
	jmp $				; hang

ds_magic:
	dd DS_MAGIC
stack_pointer:	
	dd STACK_POINTER_ADDRESS;
cpu_count:
	dd 0
cpu_lock:
	dd 0
str_cpu_is_unusable:
	db 'CPU %d is unusable because '
	db 'this kernel supports number of CPUs up to %d', CR, 0
;---------------------------------------------------------------
; GDT
;---------------------------------------------------------------
; Intel IA32 Software Developers Manual says that Aligning GDT
; to 8 byte gives performance benefits.
;
ALIGN 8	
gdt:					; adress of GDT	

gdt_null:				; Null Segment 
	dd 0				; The first Segment is reserved by CPU
	dd 0				; Initialize it with 0

KERNEL_CS 	equ	$-gdt
	dw 0FFFFh			; Limit 0:15
	dw 0				; Base 0:15
	
	db 0				; Base 16:23
	db 10011010b		; [8-12] Type
						;	[8]   - 0 : access by CPU flag
						;	[9]   - 1 : readability of this segment
						;	[10]  - 0 : conforming 
						;				( if set, allow unprivileged access )
						;   [11]  - 1 : code(1), data(0)
						;   [12]  - 1 : code or data (1), otherwise (0)
						; [13-14] - 00 : privileged level
						; [15]    - 1 : present in memory

	db 11001111b		; [16-19] - 1111 : Limit 16:19
						; [20]    - 0 : reserved by OS
						; [21]	  - 0 : reserved by CPU
						; [22]    - 1 : 32bit (1), 16bit(0)
						; [23]    - 1 : Granuality - 4KB(1), 1byte(0)
	
	db 0				; Base 24:31
	
KERNEL_DS 	equ	$-gdt
	dw 0FFFFh			; Limit 0:15
	dw 0				; Base 0:15
	
	db 0				; Base 16:23
	db 10010010b		; [8-12] Type
						;	[8]   - 0 : access by CPU flag
						;	[9]   - 1 : writablity of this segment
						;	[10]  - 0 : expand direction - up(1), down(0)
						;   [11]  - 1 : code(1), data(0)
						;   [12]  - 1 : code or data (1), otherwise (0)
						; [13-14] - 00 : privileged level
						; [15]    - 1 : present in memory
						
	db 11001111b		; same to PMODE_CS
	
	db 0				; Base 24:31
gdt_end:

; Intel IA32 Software Developers Manual says that Aligning GDT descriptor
; to 4 byte gives performance benefits.
ALIGN 4
gdt_desc:					; The GDT descriptor
	dw gdt_end - gdt - 1	; Limit - size of GDT
	dd gdt					; Adress of the GDT
