#include <includes.h>
#include "md.h"
#include "i8259.h"

/*
 * This contains the irq mask for both 8259A irq controllers,
 * 0xffff means every IRQ is disabled
 */

unsigned int cached_irq_mask = 0xffff;


/* Remapping 8259 PIC
 */

void remap_8259A_irq(u32 pic_master_irq_start_num, u32 pic_slave_irq_start_num )
{
	/* send ICW1 */
	md_out( PIC_MASTER_CMD, ICW1);
	md_out( PIC_SLAVE_CMD, ICW1);
	
	/* send ICW2 */
	/* This remaps PICS */
	md_out( PIC_MASTER_IMR, pic_master_irq_start_num );
	md_out( PIC_SLAVE_IMR, pic_slave_irq_start_num );
	
	/* send ICW3 */
	/* This is IRQ# for PICs to communicate each other */
	md_out( PIC_MASTER_IMR, 4 );
	md_out( PIC_SLAVE_IMR, 2 );
	
	/* send ICW4 */
	md_out( PIC_MASTER_IMR, ICW4);
	md_out( PIC_SLAVE_IMR, ICW4);
	
	/* disable every IRQ */
	md_out( PIC_MASTER_IMR, cached_master_mask );
	md_out( PIC_SLAVE_IMR, cached_slave_mask );
}
  
void disable_8259A_irq(u32 irq)
{ 
    u32 mask = 1 << irq;
  
//  spin_lock_irqsave(&i8259A_lock, flags);

    cached_irq_mask |= mask;

    if (irq & 8)
        md_out(cached_slave_mask, PIC_SLAVE_IMR);
    else
        md_out(cached_master_mask, PIC_MASTER_IMR);

//  spin_unlock_irqrestore(&i8259A_lock, flags);

}

void enable_8259A_irq(u32 irq)
{
    u32 mask = ~(1 << irq);

//  spin_lock_irqsave(&i8259A_lock, flags);

    cached_irq_mask &= mask;
    if (irq & 8)
        md_out(cached_slave_mask, PIC_SLAVE_IMR);
    else
        md_out(cached_master_mask, PIC_MASTER_IMR);
        
//  spin_unlock_irqrestore(&i8259A_lock, flags);
}
