#include <includes.h>
#include "md.h"
#include "intr.h"

atomic_t irq_err_count;

#define INTR_P_1	( 0x8000 )
#define INTR_DPL_0  ( 0x0000 )
#define INTR_DPL_1  ( 0x2000 )
#define INTR_DPL_2  ( 0x4000 )
#define INTR_DPL_3  ( 0x6000 )
#define INTR_D_32	( 0x0800 )
#define INTR_GATE	( 0x0600 )

typedef struct idt_gate_t {
	u16 offset_low;
	u16 seg_selector;
	u16 type;
	u16 offset_high;
} idt_gate_t ;

void set_intr_vector( u32 int_no, u32 int_proc_addr, u16 type )
{
	extern idt_gate_t idt[];
	
	idt_gate_t * ptr_idt = ( idt_gate_t * ) & idt;

	ptr_idt[ int_no ].offset_low   = (u16) ( int_proc_addr & 0xFFFF );
	ptr_idt[ int_no ].seg_selector = KERNEL_CS;
	ptr_idt[ int_no ].type		   = type;
	ptr_idt[ int_no ].offset_high  = (u16) ( int_proc_addr >> 16 );
}

void set_intr_gate( u32 int_no, u32 int_proc_addr )
{
	set_intr_vector( 
		int_no, 
		int_proc_addr, 
		INTR_P_1 | INTR_DPL_0 | INTR_D_32 | INTR_GATE );
}

#define SET_INTR_GATE( int_num, int_addr )		\
extern void int_addr(void);						\
set_intr_gate( int_num, (u32) int_addr );

void set_intr_gates()
{
	SET_INTR_GATE( 0x00, isr_cpu_00_divide_error );
    SET_INTR_GATE( 0x01, isr_cpu_01_debug );
    SET_INTR_GATE( 0x02, isr_cpu_02_nmi_interrupt );
    SET_INTR_GATE( 0x03, isr_cpu_03_breakpoint );
    SET_INTR_GATE( 0x04, isr_cpu_04_overflow );
    SET_INTR_GATE( 0x05, isr_cpu_05_bound_range_exceeded );
    SET_INTR_GATE( 0x06, isr_cpu_06_invalid_opcode );
    SET_INTR_GATE( 0x07, isr_cpu_07_device_not_available );
    SET_INTR_GATE( 0x08, isr_cpu_08_double_fault );
    SET_INTR_GATE( 0x09, isr_cpu_09_coprocessor_segment_overrun );
    SET_INTR_GATE( 0x0A, isr_cpu_0A_invalid_tss );
    SET_INTR_GATE( 0x0B, isr_cpu_0B_segment_not_present );
    SET_INTR_GATE( 0x0C, isr_cpu_0C_stack_fault );
    SET_INTR_GATE( 0x0D, isr_cpu_0D_general_protection );
    SET_INTR_GATE( 0x0E, isr_cpu_0E_page_fault );
// isr 15 is undocumented
    SET_INTR_GATE( 0x10, isr_cpu_10_x87_fpu_floating_point_error );
    SET_INTR_GATE( 0x11, isr_cpu_11_alignment_check );
    SET_INTR_GATE( 0x12, isr_cpu_12_machine_check );
    SET_INTR_GATE( 0x13, isr_cpu_13_simd_floating_point );

	SET_INTR_GATE( 0x20, isr_irq_20_system_timer );
	SET_INTR_GATE( 0x21, isr_irq_21_keyboard );
	SET_INTR_GATE( 0x22, isr_irq_22_req_from_slave_8259 );
	SET_INTR_GATE( 0x23, isr_irq_23_com2 );
	SET_INTR_GATE( 0x24, isr_irq_24_com1 );
	SET_INTR_GATE( 0x25, isr_irq_25_lpt2 );
	SET_INTR_GATE( 0x26, isr_irq_26_floppy_disk );
	SET_INTR_GATE( 0x27, isr_irq_27_lpt1 );
	SET_INTR_GATE( 0x29, isr_irq_29_vga );
	SET_INTR_GATE( 0x2C, isr_irq_2C_mouse );
	SET_INTR_GATE( 0x2D, isr_irq_2D_coprocessor );
	SET_INTR_GATE( 0x2E, isr_irq_2E_hard_disk );
}
