#include <includes.h>
#include <console.h>
#include <thread.h>

#include "md.h"
#include "thread.h"
#include "smp/mpparse.h"
#include "smp/apic.h"

machine_t mach_;


OS_INLINE u8 md_in( u16 port )
{
	u8 ret_val;
	__asm__ __volatile__(
		"in %%dx,%%al" : "=a"(ret_val) : "d"(port) 
	);

	return ret_val;
}

OS_INLINE void md_out( u16 port, u8 data )
{
	__asm__ __volatile__(
		"out %%al,%%dx" : : "a"(data),"d"(port)
	);
}


void halt() {
	for(;;);
}


// Enable the address line 20.
//
// The address line 20 must be under the control of the CPU (not always 0 as it is 
// in real-mode). This is done by sending commands to the keyboard controller (i8042). 
// Before sending commands, the i8042 must be polled to ensure that it can receive data.

void init_cpu_A20()
{
	u8	status;
	
	// Wait until i8042 can receive the command.
	do 
	{
		status = md_in(0x64);
	} while (status & 2);

	// Send the 'write' command to the i8042.
	md_out(0x64, 0xd1);

	// Wait until i8042 can receive the command.
	do 
	{
		status = md_in(0x64);
	} while (status & 2);

	// Send the new set-up.
	md_out(0x60, 0xdf);

	// Wait until i8042 has received the command.
	do 
	{
		status = md_in(0x64);
	} while (status & 2);
}

	/* Test if A20 line works correctly */

void test_cpu_A20()
{
	u32 * ptr_data1 = (u32*) 0x00000000, org_data1;
	u32 * ptr_data2 = (u32*) 0x00100000, org_data2;

	org_data1 = *ptr_data1;
	org_data2 = *ptr_data2;
	
	*ptr_data1 = 1;
	*ptr_data2 = 2;
	
	if ( *ptr_data1 == *ptr_data2 ) {
		panic( "failed to test A20 line" );
	}
	*ptr_data1 = org_data1;
	*ptr_data2 = org_data2;
}

/*
unsigned int reg_eax;
unsigned int reg_edx;

int test_cpuid(void) {
  printk("Checking for local APIC... \n");
  asm("movl $1, %eax");
  asm("cpuid");
  asm("movl %eax, reg_eax");
  asm("movl %edx, reg_edx");
  reg_edx&(1<<9) ? printk("found\n") : printk("not found\n");
  printk("(eax = 0x%x, edx = 0x%x)\n", reg_eax, reg_edx);
  return 1;
}
*/

