		[BITS 32]

	%include 'md/i686/defs.inc'

;---------------------------------------------------------------
; global, extern ..
;---------------------------------------------------------------

	global idt, init_idt
	extern set_intr_gates, con_puts, printk
	
SECTION .text 

;---------------------------------------------------------------
; body
;---------------------------------------------------------------

proc init_idt
		call set_intr_gates
		lidt [ idt_desc ]
endproc

proc md_spin_trylock
%$ptr_lock_var   arg
	push ebx
		mov eax, 1
		mov ebx, dword [ ebp +  %$ptr_lock_var ]
		lock
		xchg dword [ebx], eax
	pop ebx
endproc

extern con_put_U32

proc md_spin_lock
%$ptr_lock_var  arg
	push eax
	push ebx
		mov ebx, dword [ ebp +  %$ptr_lock_var ]
retry_lock :
		mov eax, 1
		lock
		xchg dword [ebx], eax
		cmp eax, 0
		je locked
			
		; Intel IA32 Manual says that pause instruction in spin-loop
		; helps to solve performance degrade in Hyper Threading .
		pause
		jmp retry_lock
locked:
	pop ebx
	pop eax
endproc

proc md_spin_unlock
%$ptr_lock_var   arg
	push eax
	push ebx
		mov eax, 0
		mov ebx, dword [ ebp + %$ptr_lock_var ]
		xchg dword [ebx], eax
	pop ebx
	pop eax
endproc


;proc md_arg_test3
;%$str	arg 
;%$j		arg 4
;%$i		arg 8
;	push dword [ebp + %$i]
;	xor  eax,  eax
;	mov  ax,   word[ ebp + %$j]
;	push eax
;	push dword [ebp + %$str]
;	
;	call printk
;	add  eax, 12
;endproc


%imacro dump_stack 0
	; first word
	push dword str_dumping_stack
	call printk
	add  esp, 4
	
	push dword [esp]
	push dword str_dump_eip
	call printk
	add  esp, 8
	
	push dword [esp + 4 ]
	push dword str_dump_cs
	call printk
	add  esp, 8
	
	push dword [esp + 8 ]
	push dword str_dump_eflags
	call printk
	add  esp, 8
	
	; second word
	jmp $
%endmacro

str_dumping_stack:	db '==Dumping stack==',CR,0
str_dump_eip:		db 'EIP    : 0x%x',CR,0
str_dump_cs:		db 'CS     : 0x%x',CR,0
str_dump_eflags:	db 'EFLAGS : 0x%x',CR,0


global isr_cpu_06_invalid_opcode
isr_cpu_06_invalid_opcode :
	push dword isr_name_06_invalid_opcode
	call con_puts
	add	 esp, byte 4
	
	dump_stack
	
isr_name_06_invalid_opcode :
	db 'isr_cpu_06_invalid_opcode',CR,0

;	General Protection(GP) Exception Handler
;	When CPU raises GP, it pushes a 32bit error code.
	
;	Bit 0     : EXT
;	Bit 1     : IDT
;	Bit 2     : TI
;	Bit 3 -15 : Selector Index
;	Bit 16-31 : Reserved


global isr_cpu_0D_general_protection
isr_cpu_0D_general_protection:
	pop eax			; GP exception code
	
	; push code.ti 
	mov	ebx, eax	
	shr ebx, 2
	and ebx, 01h	
	push ebx
	
	; push code.idt
	mov ebx, eax
	shr ebx, 1
	and ebx, 01h
	push ebx
	
	; push code.ext
	mov ebx, eax
	and ebx, 01h
	push ebx

	; push code.selector_index
	mov ebx, eax
	shr ebx, 3
	push ebx
	
	; push code
	push eax
	
	; push error string
	push dword str_general_protection
	
	call printk
	add  esp, 24		; pop off 6 arguments
	
	dump_stack

str_general_protection:	
db 'General protection exception. ( code = %x )',CR
db 'EXT - GP from external module like some hardware',CR
db 'IDT - GP from IDT',CR
db 'TI  - GP from LDT( if IDT==0 && TI==1 )',CR
db '      GP from GDT( if IDT==0 && TI==0 )',CR,CR
db 'Selector Index (IDT/GDT/LDT index) : 0x%x',CR
db 'EXT : %d',CR
db 'IDT : %d',CR
db 'TI : %d',CR,0

;---------------------------------------------------------------
; macro for isr
;---------------------------------------------------------------
%imacro isr_begin 0
	pushad
%endmacro

%imacro isr_end 0
	popad
	iret
%endmacro

%imacro isr_define 1
extern %1_
global %1 
%1:
	ISR_BEGIN
	
	call %1_
	
	ISR_END
%endmacro


%imacro isr_cpu 2			; macro for isr 0x00 - 0x1F

global isr_cpu_%1
isr_cpu_%1:

	ISR_BEGIN
	
;	putchar 'C'

	push dword isr_name_%1
	call con_puts
	add	 esp, byte 4
	
	ISR_END
	
isr_name_%1:
	db %2, CR, 0
%endmacro


%imacro isr_irq_master 2			; macro for isr 0x20 - 0x27

global isr_irq_%1
isr_irq_%1:

	ISR_BEGIN
	
;	putchar 'D'
	
	push dword isr_name_%1
	call con_puts
	add	 esp, byte 4
	
	; send EOI to master 8259
	mov	 al, 020h
	out  020h, al
	
	ISR_END
	
isr_name_%1:
	db %2, CR, 0
%endmacro



%imacro isr_irq_slave 2			; macro for isr 0x28 - 0x2F

global isr_irq_%1			
isr_irq_%1:
	ISR_BEGIN

	; display a blinking white-on-blue character
;	putchar 'E'

	
	push dword isr_name_%1
	call con_puts
	add	 esp, byte 4
	
	; Send EOI to both master and slave PIC
	mov  al, 020h
	out  020h, al
	out	 0A0h, al
	
	ISR_END
	
isr_name_%1:
	db %2, CR, 0
%endmacro

ISR_CPU 00_divide_error,					'isr_cpu_00_divide_error'
ISR_CPU 01_debug,							'isr_cpu_01_debug'
ISR_CPU 02_nmi_interrupt,					'isr_cpu_02_nmi_interrupt'
ISR_CPU 03_breakpoint,						'isr_cpu_03_breakpoint'
ISR_CPU 04_overflow,						'isr_cpu_04_overflow'
ISR_CPU 05_bound_range_exceeded,			'isr_cpu_05_bound_range_exceeded'
;ISR_CPU 06_invalid_opcode,					'isr_cpu_06_invalid_opcode'
ISR_CPU 07_device_not_available,			'isr_cpu_07_device_not_available'
ISR_CPU 08_double_fault,					'isr_cpu_08_double_fault'
ISR_CPU 09_coprocessor_segment_overrun,		'isr_cpu_09_coprocessor_segment_overrun'
ISR_CPU 0A_invalid_tss,						'isr_cpu_0A_invalid_tss'
ISR_CPU 0B_segment_not_present,				'isr_cpu_0B_segment_not_present'
ISR_CPU 0C_stack_fault,						'isr_cpu_0C_stack_fault'
; ISR_CPU 0D_general_protection,				'isr_cpu_0D_general_protection'
ISR_CPU 0E_page_fault,						'isr_cpu_0E_page_fault'
; isr 15 is undocumented
ISR_CPU 10_x87_fpu_floating_point_error,	'isr_cpu_10_x87_fpu_floating_point_error'
ISR_CPU 11_alignment_check,					'isr_cpu_11_alignment_check'
ISR_CPU 12_machine_check,					'isr_cpu_12_machine_check'
ISR_CPU 13_simd_floating_point,				'isr_cpu_13_simd_floating_point'

;ISR_IRQ_MASTER		20_system_timer,		'isr_irq_20_system_timer'
ISR_IRQ_MASTER		21_keyboard,			'isr_irq_21_keyboard'
ISR_IRQ_MASTER		22_req_from_slave_8259,	'isr_irq_22_req_from_slave_8259'
ISR_IRQ_MASTER		23_com2,				'isr_irq_23_com2'
ISR_IRQ_MASTER		24_com1,				'isr_irq_24_com1'
ISR_IRQ_MASTER		25_lpt2,				'isr_irq_25_lpt2'
ISR_IRQ_MASTER		26_floppy_disk,			'isr_irq_26_floppy_disk'
ISR_IRQ_MASTER		27_lpt1,				'isr_irq_27_lpt1'
;ISR_IRQ_SLAVE		28_cmos_clock,			'isr_irq_28_cmos_clock'
ISR_IRQ_SLAVE		29_vga,					'isr_irq_29_vga'
ISR_IRQ_SLAVE		2C_mouse,				'isr_irq_2C_mouse'
ISR_IRQ_SLAVE		2D_coprocessor,			'isr_irq_2D_coprocessor'
ISR_IRQ_SLAVE		2E_hard_disk,			'isr_irq_2E_hard_disk'

; do nothing for now
global isr_irq_20_system_timer
isr_irq_20_system_timer :
	ISR_BEGIN
	
	putchar 'D'
	
;	push dword isr_name_%1
;	call con_puts
;	add	 esp, byte 4
	
	; send EOI to master 8259
	mov	 al, 020h
	out  020h, al
	
	ISR_END

ISR_DEFINE apic_timer_interrupt
ISR_DEFINE spurious_interrupt
ISR_DEFINE error_interrupt

;---------------------------------------------------------------
; IDT
;---------------------------------------------------------------
; First initialize IDTs with 0, which sets present bit to zero.
; When present bit is zero, interrupt will not occur.
;
ALIGN 8
idt:
	times 64 dd	0 ; [ 00 - 1F ] Intel Reserved
	times 32 dd 0 ; [ 20 - 2F ] IRQs ( by remapping 8259 )
	times 32 dd 0 ; [ 30 - 3F ] LightOS Reserved
idt_end:

ALIGN 4
idt_desc:
	dw idt_end - idt - 1
	dd idt
	
