#ifndef _PROFILER_H_
#define _PROFILER_H_ 1

#include <includes.h>
#include <console.h>
#include "msr.h"

#define MAX_PROFILES 50

typedef struct profiler_t
{
	u64 tscs[MAX_PROFILES];
	s32 * cp_name [ MAX_PROFILES ] ;
	u32 tscs_count;
	u32 is_on;
} profiler_t ;

#define prof_init( pf )	\
{						\
	pf.tscs_count = 0;	\
	pf.is_on	  = 1;	\
}

#define prof_on( pf )	\
{						\
	pf.is_on = 1;		\
}

#define prof_off( pf )	\
{						\
	pf.is_on = 0;		\
}


#define prof_checkpoint( pf, name )					\
{													\
	if ( pf.is_on )									\
	{												\
		pf.cp_name[ pf.tscs_count ] = name 	;		\
		rdtscll( pf.tscs [ pf.tscs_count ++ ] );	\
	}												\
}

#define prof_print( pf )													\
{																			\
	u32 i;																	\
	u64 ts_curr_diff ;														\
	u64 ts_first = pf.tscs[ 0 ];											\
	u64 ts_last  = pf.tscs[ pf.tscs_count -1 ];								\
																			\
	printk( "==listing profile==\n", i, pf.tscs[ i ] - pf.tscs[ 0 ] );		\
	for ( i=0; i < pf.tscs_count ; i++ )									\
	{	if ( i == 0 ) 														\
		{																	\
			ts_curr_diff = 0;												\
		}																	\
		else																\
		{																	\
			ts_curr_diff = pf.tscs[ i ] - pf.tscs[ i - 1 ];					\
		}																	\
		printk( "[%2d][%3lu%\%][%s] %llu\n", 									\
			i,																\
			(u32)( (u32)( ts_curr_diff * 100 ) / (u32)( ts_last - ts_first) ),	\
			pf.cp_name[ i ],												\
			ts_curr_diff );													\
	}																		\
}

extern profiler_t default_profiler;

#define PROF_INIT() 		prof_init( default_profiler );
#define PROF_ON()			prof_on( default_profiler );
#define PROF_OFF()			prof_off( default_profiler );

#define PROF_CHECKPOINT(name)	prof_checkpoint( default_profiler, name );
#define PROF_PRINT()		prof_print( default_profiler );

#endif /* _PROFILER_H_ */
