/* include/asm-i386/rwlock.h
 *
 *      Helpers used by both rw spinlocks and rw semaphores.
 *
 *      Based in part on code from semaphore.h and
 *      spinlock.h Copyright 1996 Linus Torvalds.
 *
 *      Copyright 1999 Red Hat, Inc.
 *
 *      Written by Benjamin LaHaise.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 */
 
#ifndef _RWLOCK_H_
#define _RWLOCK_H_

/*
 * Must define these before including other files, inline functions need them
 */
//#define LOCK_SECTION_NAME                       \
//        ".text.lock." __stringify(KBUILD_BASENAME)
#define LOCK_SECTION_NAME                       \
        ".text.lock." "a"


#define LOCK_SECTION_START(extra)               \
        ".subsection 1\n\t"                     \
        extra                                   \
        ".ifndef " LOCK_SECTION_NAME "\n\t"     \
        LOCK_SECTION_NAME ":\n\t"               \
        ".endif\n\t"
#define LOCK_SECTION_END                        \
        ".previous\n\t"


#define RW_LOCK_BIAS             0x01000000
#define RW_LOCK_BIAS_STR        "0x01000000"

#define __build_read_lock_ptr(rw, helper)   \
        asm volatile(LOCK "subl $1,(%0)\n\t" \
                     "js 2f\n" \
                     "1:\n" \
                     LOCK_SECTION_START("") \
                     "2:\tcall " helper "\n\t" \
                     "jmp 1b\n" \
                     LOCK_SECTION_END \
                     ::"a" (rw) : "memory")

#define __build_read_lock_const(rw, helper)   \
        asm volatile(LOCK "subl $1,%0\n\t" \
                     "js 2f\n" \
                     "1:\n" \
                     LOCK_SECTION_START("") \
                     "2:\tpushl %%eax\n\t" \
                     "leal %0,%%eax\n\t" \
                     "call " helper "\n\t" \
                     "popl %%eax\n\t" \
                     "jmp 1b\n" \
                     LOCK_SECTION_END \
                     :"=m" (*(volatile int *)rw) : : "memory")

#define __build_read_lock(rw, helper)   do { \
                                                if (__builtin_constant_p(rw)) \
                                                        __build_read_lock_const(rw, helper); \
                                                else \
                                                        __build_read_lock_ptr(rw, helper); \
                                        } while (0)

#define __build_write_lock_ptr(rw, helper) \
        asm volatile(LOCK "subl $" RW_LOCK_BIAS_STR ",(%0)\n\t" \
                     "jnz 2f\n" \
                     "1:\n" \
                     LOCK_SECTION_START("") \
                     "2:\tcall " helper "\n\t" \
                     "jmp 1b\n" \
                     LOCK_SECTION_END \
                     ::"a" (rw) : "memory")

#define __build_write_lock_const(rw, helper) \
        asm volatile(LOCK "subl $" RW_LOCK_BIAS_STR ",%0\n\t" \
                     "jnz 2f\n" \
                     "1:\n" \
                     LOCK_SECTION_START("") \
                     "2:\tpushl %%eax\n\t" \
                     "leal %0,%%eax\n\t" \
                     "call " helper "\n\t" \
                     "popl %%eax\n\t" \
                     "jmp 1b\n" \
                     LOCK_SECTION_END \
                     :"=m" (*(volatile int *)rw) : : "memory")

#define __build_write_lock(rw, helper)  do { \
                                                if (__builtin_constant_p(rw)) \
                                                        __build_write_lock_const(rw, helper); \
                                                else \
                                                        __build_write_lock_ptr(rw, helper); \
                                        } while (0)

#endif /* _RWLOCK_H_ */
