#ifndef _APIC_H_
#define _APIC_H_ 1

#include <type.h>
#include <thread.h>
#include "../md.h"
#include "apic_def.h"

/* APIC Register base address for currently running thread */
#define APIC_BASE  ( (u8*) ( mach_.cpu[ thr_cpu_id() ].lapic ) )

static __inline void apic_write(unsigned long reg, unsigned long v)
{
        *((volatile unsigned long *)(APIC_BASE+reg)) = v;
}

static __inline void apic_write_atomic(unsigned long reg, unsigned long v)
{
        xchg((volatile unsigned long *)(APIC_BASE+reg), v);
}

static __inline unsigned long apic_read(unsigned long reg)
{
        return *((volatile unsigned long *)(APIC_BASE+reg));
}

static __inline__ void apic_wait_icr_idle(void)
{
        do { } while ( apic_read( APIC_ICR ) & APIC_ICR_BUSY );
}

#ifdef CONFIG_X86_GOOD_APIC
# define FORCE_READ_AROUND_WRITE 0
# define apic_read_around(x)
# define apic_write_around(x,y) apic_write((x),(y))
#else
# define FORCE_READ_AROUND_WRITE 1
# define apic_read_around(x) apic_read(x)
# define apic_write_around(x,y) apic_write_atomic((x),(y))
#endif

static inline void ack_APIC_irq(void)
{
        /*
         * ack_APIC_irq() actually gets compiled as a single instruction:
         * - a single rmw on Pentium/82489DX
         * - a single write on P6+ cores (CONFIG_X86_GOOD_APIC)
         * ... yummie.
         */

        /* Docs say use 0 for future compatibility */
        apic_write_around(APIC_EOI, 0);
}


extern void init_apic_intr(void);

extern void map_lapic(struct cpu_info_t * c);

extern void setup_8254_timer();

#endif
