#ifndef _MPSPEC_DEF_H
#define _MPSPEC_DEF_H

/*
 * Structure definitions for SMP machines following the
 * Intel Multiprocessing Specification 1.1 and 1.4.
 */

/*
 * This tag identifies where the SMP configuration
 * information is. 
 */
 
#define SMP_MAGIC_IDENT	(('_'<<24)|('P'<<16)|('M'<<8)|'_')

#define MAX_MPC_ENTRY 1024

struct intel_mp_floating
{
	s8 mpf_signature[4];/* "_MP_" 			*/
	u32 mpf_physptr;	/* Configuration table address	*/
	u8 mpf_length;	/* Our length (paragraphs)	*/
	u8 mpf_specification;/* Specification version	*/
	u8 mpf_checksum;	/* Checksum (makes sum 0)	*/
	u8 mpf_feature1;	/* Standard or configuration ? 	*/
	u8 mpf_feature2;	/* Bit7 set for IMCR|PIC	*/
	u8 mpf_feature3;	/* Unused (0)			*/
	u8 mpf_feature4;	/* Unused (0)			*/
	u8 mpf_feature5;	/* Unused (0)			*/
};

struct mp_config_table
{
	s8 mpc_signature[4];
#define MPC_SIGNATURE "PCMP"
	u16 mpc_length;	/* Size of table */
	s8  mpc_spec;			/* 0x01 */
	s8  mpc_checksum;
	s8  mpc_oem[8];
	s8  mpc_productid[12];
	u32 mpc_oemptr;	/* 0 if not present */
	u16 mpc_oemsize;	/* 0 if not present */
	u16 mpc_oemcount;
	u32 mpc_lapic;	/* APIC address */
	u32 reserved;
};

/* Followed by entries */

#define	MP_PROCESSOR	0
#define	MP_BUS		1
#define	MP_IOAPIC	2
#define	MP_INTSRC	3
#define	MP_LINTSRC	4
#define	MP_TRANSLATION  192  /* Used by IBM NUMA-Q to describe node locality */

struct mpc_config_processor
{
	u8 mpc_type;
	u8 mpc_apicid;	/* Local APIC number */
	u8 mpc_apicver;	/* Its versions */
	u8 mpc_cpuflag;
#define CPU_ENABLED		1	/* Processor is available */
#define CPU_BOOTPROCESSOR	2	/* Processor is the BP */
	u32 mpc_cpufeature;		
#define CPU_STEPPING_MASK 0x0F
#define CPU_MODEL_MASK	0xF0
#define CPU_FAMILY_MASK	0xF00
	u32 mpc_featureflag;	/* CPUID feature value */
	u32 mpc_reserved[2];
};

struct mpc_config_bus
{
	u8 mpc_type;
	u8 mpc_busid;
	u8 mpc_bustype[6] __attribute((packed));
};

/* List of Bus Type string values, Intel MP Spec. */
#define BUSTYPE_EISA	"EISA"
#define BUSTYPE_ISA	"ISA"
#define BUSTYPE_INTERN	"INTERN"	/* Internal BUS */
#define BUSTYPE_MCA	"MCA"
#define BUSTYPE_VL	"VL"		/* Local bus */
#define BUSTYPE_PCI	"PCI"
#define BUSTYPE_PCMCIA	"PCMCIA"
#define BUSTYPE_CBUS	"CBUS"
#define BUSTYPE_CBUSII	"CBUSII"
#define BUSTYPE_FUTURE	"FUTURE"
#define BUSTYPE_MBI	"MBI"
#define BUSTYPE_MBII	"MBII"
#define BUSTYPE_MPI	"MPI"
#define BUSTYPE_MPSA	"MPSA"
#define BUSTYPE_NUBUS	"NUBUS"
#define BUSTYPE_TC	"TC"
#define BUSTYPE_VME	"VME"
#define BUSTYPE_XPRESS	"XPRESS"
#define BUSTYPE_NEC98	"NEC98"

struct mpc_config_ioapic
{
	u8 mpc_type;
	u8 mpc_apicid;
	u8 mpc_apicver;
	u8 mpc_flags;
#define MPC_APIC_USABLE		0x01
	u32 mpc_apicaddr;
};

struct mpc_config_intsrc
{
	u8 mpc_type;
	u8 mpc_irqtype;
	u16 mpc_irqflag;
	u8 mpc_srcbus;
	u8 mpc_srcbusirq;
	u8 mpc_dstapic;
	u8 mpc_dstirq;
};

enum mp_irq_source_types {
	mp_INT = 0,
	mp_NMI = 1,
	mp_SMI = 2,
	mp_ExtINT = 3
};

#define MP_IRQDIR_DEFAULT	0
#define MP_IRQDIR_HIGH		1
#define MP_IRQDIR_LOW		3


struct mpc_config_lintsrc
{
	u8 mpc_type;
	u8 mpc_irqtype;
	u16 mpc_irqflag;
	u8 mpc_srcbusid;
	u8 mpc_srcbusirq;
	u8 mpc_destapic;	
#define MP_APIC_ALL	0xFF
	u8 mpc_destapiclint;
};

struct mp_config_oemtable
{
	s8  oem_signature[4];
#define MPC_OEM_SIGNATURE "_OEM"
	u16 oem_length;	/* Size of table */
	s8  oem_rev;			/* 0x01 */
	s8  oem_checksum;
	s8  mpc_oem[8];
};

struct mpc_config_translation
{
	u8 mpc_type;
	u8 trans_len;
	u8 trans_type;
	u8 trans_quad;
	u8 trans_global;
	u8 trans_local;
	u16 trans_reserved;
};

/*
 *	Default configurations
 *
 *	1	2 CPU ISA 82489DX
 *	2	2 CPU EISA 82489DX neither IRQ 0 timer nor IRQ 13 DMA chaining
 *	3	2 CPU EISA 82489DX
 *	4	2 CPU MCA 82489DX
 *	5	2 CPU ISA+PCI
 *	6	2 CPU EISA+PCI
 *	7	2 CPU MCA+PCI
 */

enum mp_bustype {
	MP_BUS_ISA = 1,
	MP_BUS_EISA,
	MP_BUS_PCI,
	MP_BUS_MCA,
	MP_BUS_NEC98
};



#endif /* _MPSPEC_DEF_H */

